import argparse
import pathlib
import sys

from . import linting
from . import utils


def get_arguments() -> argparse.Namespace:
    """Parse user-provided CLI arguments.

    A user can change the default utility behavior.
    This method used to parse and prepare arguments.

    Returns:
        A parsed user-provided CLI arguments.
    """
    parser = argparse.ArgumentParser(description='Copyright linting utility')

    parser.add_argument(
        '-w',
        '--workdir',
        default=pathlib.Path.cwd(),
        help='Root directory to start copyrighting checking',
        type=pathlib.Path)

    return parser.parse_args()


def main() -> None:
    """Program entry point function."""
    try:
        arguments = get_arguments()
        config = utils.get_config(arguments.workdir)
        linting.lint_project(arguments.workdir, config.ignores)
    except Exception as e:
        sys.exit(e)


if __name__ == '__main__':
    main()



import typing


class LinterException(Exception):
    """The base exception class for all library raises."""


class CopyrightException(LinterException):
    """When the copyright of a file is in poor condition."""

    def __init__(self, errors: typing.Any) -> None:
        if isinstance(errors, (list, tuple)):
            # Convert pathlib.Path to list of objects for beautify print.
            errors = '\n '.join((str(error.absolute()) for error in errors))

        super().__init__('Failed linting in files:\n {0}'.format(errors))



import pathlib
import re
import typing

from . import exceptions
from . import utils


# Copyright re-strings template.
COPYRIGHT_TEMPLATE = [
    r"Copyright \(c\) 20\d{2} Celadon Development LLC, All rights reserved.",
    r"Author \w+ \w+ <\w+\.\w+@celadon.ae>",
]


def _build_copyright(copyright_lines: typing.List[str], comment_symbol: str) -> str:
    """Fill copyright template with comment symbols.

    For each language used in the project its own comment
    symbols. This method used to fill copyright string
    template with comment symbol for a specific language.

    Arguments:
        copyright_lines: a copyright strings template.
        comment_symbol: a symbol what used for commenting lines.

    Returns:
        A filled copyright template with comment symbol.
    """
    builded_copyright_lines = [
        ' '.join([comment_symbol, line])
        for line in copyright_lines
    ]
    return '\n'.join(builded_copyright_lines)


# Match file type with it is comment symbols by extension.
COPYRIGHT_BY_FILE_EXTENSION = {
    'js': _build_copyright(COPYRIGHT_TEMPLATE, '//'),
    'py': _build_copyright(COPYRIGHT_TEMPLATE, '#'),
    'yml': _build_copyright(COPYRIGHT_TEMPLATE, '#'),
}


def get_copyright_by_extension(file_path: str) -> str:
    """Get copyright by file extension.

    This method used to get the copyright string
    template for a specific language.

    Arguments:
        file_path: a file path to be inspected.

    Returns:
        A filled copytight re-string pattern.
    """
    extension = pathlib.Path(file_path).suffixes[-1].lstrip('.')
    return COPYRIGHT_BY_FILE_EXTENSION[extension]


def lint_file_copyright(file_path: str) -> None:
    """Check for file copyright and its status.

    All project files must contain a copyright stings.
    This method used to check the presence and status
    for copyright strings in the current file.

    Arguments:
        file_path: a path to file to be checked.

    Raises:
        CopyrightException: raises if file has broken copyright.
    """
    with pathlib.Path(file_path).open(encoding='utf-8') as file:
        file_copyright = re.compile(get_copyright_by_extension(file_path), re.MULTILINE)

        # Check copyright statements for a current file.
        if file_copyright.search(file.read()) is None:
            raise exceptions.CopyrightException(file_path)


def lint_project(work_path: pathlib.Path, exclusion: typing.List[str]) -> None:
    """Lint all files in the user-provided directory.

    This method used to check the copyright the presence
    and status for all files in user-provided project.

    Arguments:
        work_path: a root directory for running a check
        exclusion: exclusion file extension patterns.

    Raises:
        LinterException: raises if file has broken copyright.
    """
    problem_files = []
    for file_path in utils.scantree(work_path):
        if not utils.is_excluded(file_path, exclusion):
            try:
                lint_file_copyright(file_path)
            except exceptions.CopyrightException:
                problem_files.append(file_path)
            except Exception:
                pass

    # Show all broken files that have broken copyright.
    if problem_files:
        raise exceptions.CopyrightException(problem_files)



import collections
import configparser
import fnmatch
import pathlib
import typing


# Structure that identifies the configuration.
Config = collections.namedtuple('Config', [
    'ignores',
])

# Structure that identifies config types what can be parsed.
Configfile = collections.namedtuple('Configfile', [
    'filename',
    'option',
    'section',
])

# Types of config files for parsing.
CRLINT_CONFIG = Configfile(filename='.crlintrc', option='patterns', section='ignore')
SETUP_CONFIG = Configfile(filename='setup.cfg', option='patterns', section='crlint')


def scantree(work_path: str) -> typing.Iterator:
    """Directory recursive scanning.

    Directory recursive scanning for getting all files
    paths. This method used to walk directory recursively
    and return all file paths to this directory.

    Arguments:
        root_path: a provided path to start scanning.

    Yields:
        An absolute path to the current file.
    """
    for path in pathlib.Path(work_path).glob('**/*'):
        if path.is_file():
            yield path.absolute()


def is_excluded(file_path: str, exclusions: typing.List[str]) -> bool:
    """Check file path to match ignore pattern.

    Excludes file paths if they match user-submitted
    exclusion patterns. This method used to filtering
    available file paths by providing ignore patterns.

    Arguments:
        file_path: provided path to file for checking.
        exclusions: a files ignore pattern.

    Returns:
        An indicator for current file ignores it or not.
    """
    return any(
        fnmatch.fnmatch(file_path, exclusion)
        for exclusion in exclusions
    )


def get_config(work_path: str) -> configparser.ConfigParser:
    """Load configuration file and parse it.

    Depending on what config file (setup.cfg or crlitrc)
    get actual sections. If there are both files setup.cfg
    will be primarily parsed. This method use to load and
    parse configuration file.

    Arguments:
        work_path: a root directory for running a check.

    Raises:
        FileNotFoundError: raises if file wasn't found.

    Returns:
        A parsed the configuration file.
    """
    work_path = pathlib.Path(work_path)

    # Find the configuration file by priority.
    config_file = None
    for config in (SETUP_CONFIG, CRLINT_CONFIG):
        if work_path.joinpath(config.filename).exists():
            config_file = config
            break

    # Raise an error in case of not finding at least one config.
    if config_file is None:
        raise FileNotFoundError('Cannot found {0} or {1} config'.format(
            CRLINT_CONFIG.filename,
            SETUP_CONFIG.filename,
        ))

    # Loading the founded configuration file.
    config = configparser.ConfigParser()
    config.read(work_path.joinpath(config_file.filename))

    config_exclusion_patterns = [
        pattern.strip()
        for pattern in config.get(config_file.section, config_file.option).split(',')
    ]
    return Config(ignores=config_exclusion_patterns)



#!/usr/bin/env python3
import os

from setuptools import setup, find_packages


# Get long description about this python project.
with open('README.md', encoding='utf-8') as readme_file:
    long_description = readme_file.read()


setup(
    name='crlint',
    # TODO: remake package versioning.
    version=os.getenv('BITBUCKET_TAG') or os.environ['CT_LOCAL_VERSION'],
    python_requires='>=3.4',
    url='https://bitbucket.org/celadonteam/crtool',
    author='Celadon Developers',
    author_email='opensource@celadon.ae',
    description='The project copyright linter',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    scripts=['crlint/__main__.py'],
    entry_points={
        'console_scripts': [
            'crlint=crlint.__main__:main',
        ],
    },
    zip_safe=False,
    classifiers=[
        'Environment :: Console',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development',
    ],
)
